#!/bin/bash
#
# Installation Script for ACR120U Contactless Smart Card Reader Shared Library (Driver)
#
# Version 1.5.0.4 (Windows DLL 1.5.0.4)
# Advanced Card Systems Ltd
# http://www.acs.com.hk
#

#################### Variables ####################
DEVICE_NAME="ACR120U"
LIB_MJ_VER="1"				# Major Version
LIB_MN_VER="5"				# Minor Version
LIB_RV_VER="0"				# Revision
LIB_BI_VER="4"				# Build
SHARED_LIB="libACR120U.so"
HEADER_FIL="ACR120.h"
HOME_DIR=`pwd`
INSTALL_PATH=/usr/local
INSTALL_RC=~/.acr120urc

#################### Functions ####################
function print_help()
{
	echo "Usage: $0 [-h | -i | -u]"
	echo -e "\t -h\t print this help message"
	echo -e "\t -i\t install driver library package"
	echo -e "\t -u\t uninstall driver library package"
}

function install_lib()
{
	local new_install_path;

	echo -n "Please specify the location to install driver library [default: $INSTALL_PATH]: "
	read new_install_path;

	if [ -z $new_install_path ]; then
		INSTALL_PATH=$INSTALL_PATH;
	else
		INSTALL_PATH=$new_install_path;
	fi

	echo
	echo Installing ACR120U Shared Library ...
	echo
	echo Installing driver library to $INSTALL_PATH/lib ...

	install -D $SHARED_LIB.$LIB_MJ_VER.$LIB_MN_VER.$LIB_RV_VER.$LIB_BI_VER $INSTALL_PATH/lib/$SHARED_LIB.$LIB_MJ_VER.$LIB_MN_VER.$LIB_RV_VER.$LIB_BI_VER

	echo
	echo Installing library header to $INSTALL_PATH/include ...

	install -D -m 644 $HEADER_FIL $INSTALL_PATH/include/$HEADER_FIL

	ln -sf $SHARED_LIB.$LIB_MJ_VER.$LIB_MN_VER.$LIB_RV_VER.$LIB_BI_VER $INSTALL_PATH/lib/$SHARED_LIB.$LIB_MJ_VER
	ln -sf $SHARED_LIB.$LIB_MJ_VER $INSTALL_PATH/lib/$SHARED_LIB

	echo
	echo "Installation is done!"

	touch $INSTALL_RC
	echo "[Installation Path]" >> $INSTALL_RC
	echo $INSTALL_PATH >> $INSTALL_RC
}

function uninstall_lib()
{
	local install_path;

	if [ -f $INSTALL_RC ]; then

		echo
		echo "Uninstalling $DEVICE_NAME driver library ..."
		echo 

		while read line
		do
			if [ "$line"="[Installation Path]" ]; then
				read install_path;
			fi
		done < $INSTALL_RC;

		echo
		echo "Removing $SHARED_LIB.$LIB_MJ_VER.$LIB_MN_VER.$LIB_RV_VER.$LIB_BI_VER from $install_path/lib ..."
		echo
		rm -rf $install_path/lib/$SHARED_LIB.$LIB_MJ_VER.$LIB_MN_VER.$LIB_RV_VER.$LIB_BI_VER

		echo
		echo "Removing $SHARED_LIB.$LIB_MJ_VER from $install_path/lib ..."
		echo
		rm -rf $install_path/lib/$SHARED_LIB.$LIB_MJ_VER

		echo
		echo "Removing $SHARED_LIB from $install_path/lib ..."
		echo
		rm -rf $install_path/lib/$SHARED_LIB

		echo
		echo "Removing $HEADER_FIL from $install_path/include ..."
		echo
		rm -rf $install_path/include/$HEADER_FIL
		rm -rf $INSTALL_RC

		echo "Uninstallation is done!"

	else
		echo 
		echo "$DEVICE_NAME driver library has not been installed before."
		echo "Uninstallation aborted."
		echo 
	fi
}

#################### Main ####################
if [ "$1" == "-h" ] || [ "$1" == "" ]; then
	print_help
	exit 0;
fi

if [ $UID -ne "0" ]; then
	echo "You must be root in order to run this script!"
	exit -1;
fi

if [ "$1" == "-i" ]; then
	install_lib
	exit 0;
fi

if [ "$1" == "-u" ]; then
	uninstall_lib
	exit 0;
fi
